@@ -453,7 +453,10 @@ func startWalker(dir string, res chan<- fileInfo, abort <-chan struct{}) chan er
 			f = fileInfo{
 				name: rn,
 				mode: info.Mode(),
-				mod:  info.ModTime().Truncate(time.Microsecond).UnixNano(),
+				// comparing timestamps with better precision than a second
+				// is problematic as there is rounding and truncatign going
+				// on at every level
+				mod:  info.ModTime().Unix(),
 				size: info.Size(),
 			}
 			sum, err := md5file(path)
